<?php

require_once(ABSPATH . 'wp-config.php'); // DB constants for MYSQL connection
require_once(dirname(__FILE__) . '/jsonpluginexception.php'); // Custom JSON exception class

class DbConnection {
    private $configSchema = "CREATE TABLE IF NOT EXISTS `frontier_config_v1` (
        `name` VARCHAR(191) NOT NULL PRIMARY KEY, 
        `value` VARCHAR(191) NOT NULL
    )
    CHARACTER SET utf8mb4
    COLLATE utf8mb4_unicode_520_ci";

    private $blobSchema = "CREATE TABLE IF NOT EXISTS `frontier_blobs_v1` (
        `hash` VARCHAR(64) NOT NULL PRIMARY KEY, 
        `blob` LONGBLOB NOT NULL
    )
    CHARACTER SET utf8mb4
    COLLATE utf8mb4_unicode_520_ci";

    private $productSchema = "CREATE TABLE IF NOT EXISTS `frontier_products_v1` (
        `id` VARCHAR(64) NOT NULL PRIMARY KEY,
        `prefix` VARCHAR(191) NOT NULL UNIQUE,
        `title` VARCHAR(191) NOT NULL,
        `type` VARCHAR(64) NOT NULL,
        `start` BIGINT NOT NULL,
        `end` BIGINT NOT NULL
    )
    CHARACTER SET utf8mb4
    COLLATE utf8mb4_unicode_520_ci";

    private $urlSchema = "CREATE TABLE IF NOT EXISTS `frontier_urls_v1` (
        `product_id` VARCHAR(64) NOT NULL,
        `suffix` VARCHAR(191) NOT NULL,
        `mime` VARCHAR(64) NOT NULL,
        `hash` VARCHAR(64) NOT NULL,
        FOREIGN KEY (product_id) REFERENCES frontier_products_v1 (id),
        PRIMARY KEY (product_id, suffix)
    )
    CHARACTER SET utf8mb4
    COLLATE utf8mb4_unicode_520_ci";

    function __construct() {
        mysqli_report(MYSQLI_REPORT_ERROR | MYSQLI_REPORT_STRICT);
        $this->connection = mysqli_connect(DB_HOST, DB_USER, DB_PASSWORD, DB_NAME);
    }
    
    // Used to create tables when plugin is activated
    public function dbTableCreate() {
        mysqli_query($this->connection, $this->configSchema);
        mysqli_query($this->connection, $this->blobSchema);
        mysqli_query($this->connection, $this->productSchema);
        mysqli_query($this->connection, $this->urlSchema);
    }

    public function configSelect($name) {
        $sql = "SELECT `value` FROM frontier_config_v1 WHERE `name`=?";
        try {
            $stmt = mysqli_prepare($this->connection, $sql);
            mysqli_stmt_bind_param($stmt, 's', $name);
            mysqli_stmt_execute($stmt);
            $result = mysqli_stmt_get_result($stmt);
            $row = mysqli_fetch_assoc($result);
            return $row ? $row['value'] : NULL;
        } catch (Exception $e) {
            throw new JsonPluginException('db-error', $e->getMessage());
        } finally {
            mysqli_stmt_close($stmt);
        }
    }

    public function configInsert($name, $value) {
        $sql = "INSERT IGNORE INTO frontier_config_v1 (`name`, `value`) VALUES (?, ?)";
        $sql = "INSERT IGNORE INTO frontier_config_v1 (`name`, `value`) VALUES (?, ?)";
        try {
            $stmt = mysqli_prepare($this->connection, $sql);
            mysqli_stmt_bind_param($stmt, 'ss', $name, $value);
            mysqli_stmt_execute($stmt);
            return mysqli_affected_rows($this->connection);
        } catch (Exception $e) {
            throw new JsonPluginException('db-error', $e->getMessage());
        } finally {
            mysqli_stmt_close($stmt);
        }
    }

    public function blobSelect($hash) {
        $sql = "SELECT `blob` FROM frontier_blobs_v1 WHERE `hash`=?";
        try {
            $stmt = mysqli_prepare($this->connection, $sql);
            mysqli_stmt_bind_param($stmt, 's', strval($hash));
            mysqli_stmt_execute($stmt);
            $result = mysqli_stmt_get_result($stmt);
            $row = mysqli_fetch_assoc($result);
            return $row ? $row['blob'] : NULL;
        } catch (Exception $e) {
            throw new JsonPluginException('db-error', $e->getMessage());
        } finally {
            mysqli_stmt_close($stmt);
        }
    }

    public function blobInsert($hash, $blob) {
        $sql = "INSERT IGNORE INTO frontier_blobs_v1 (`hash`, `blob`) VALUES (?, ?)";
        try {
            $stmt = mysqli_prepare($this->connection, $sql);
            $null = NULL;
            mysqli_stmt_bind_param($stmt, 'sb', strval($hash), $null);

            // Put blob into in-memory stream and chunk it into DB
            $fp = fopen('php://memory', 'w+b');
            fwrite($fp, $blob);
            rewind($fp);
            while(!feof($fp)) {
                mysqli_stmt_send_long_data($stmt, 1, fread($fp, 8192));
            }
            fclose($fp);
            mysqli_stmt_execute($stmt);
        } catch (Exception $e) {
            throw new JsonPluginException('db-error', $e->getMessage());
        } finally {
            mysqli_stmt_close($stmt);
        }
    }

    public function productSelectAll() {
        $sql = "SELECT prefix, title, type, start, end FROM frontier_products_v1";
        try {
            $result = mysqli_query($this->connection, $sql);
            $rows = [];
            while($row = mysqli_fetch_assoc($result)) {
                $rows[$row['prefix']] = [
                    'title' => $row['title'],
                    'type' => $row['type'],
                    'start' => intval($row['start']), 
                    'end' => intval($row['end'])
                ];
            }
            return $rows;
        } catch (Exception $e) {
            throw new JsonPluginException('db-error', $e->getMessage());
        }
    }

    public function productInsert($id, $prefix, $title, $type, $start, $end) {
        $sql = "INSERT INTO frontier_products_v1 (id, prefix, title, type, start, end) 
                VALUES (?, ?, ?, ?, ?, ?) 
                ON DUPLICATE KEY UPDATE 
                title = VALUES(title),
                type = VALUES(type),
                start = VALUES(start),
                end = VALUES(end)";
        try {
            $stmt = mysqli_prepare($this->connection, $sql);
            mysqli_stmt_bind_param($stmt, 'ssssii', $id, $prefix, $title, $type, $start, $end);
            mysqli_stmt_execute($stmt);
            return mysqli_insert_id($this->connection);
        } catch (Exception $e) {
            throw new JsonPluginException('db-error', $e->getMessage());
        } finally {
            mysqli_stmt_close($stmt);
        }
    }

    public function productDelete($id) {
        $sql = "DELETE FROM frontier_products_v1 WHERE id=?";
        try {
            $stmt = mysqli_prepare($this->connection, $sql);
            mysqli_stmt_bind_param($stmt, 's', $id);
            return mysqli_stmt_execute($stmt);
        } catch (Exception $e) {
            throw new JsonPluginException('db-error', $e->getMessage());
        } finally {
            mysqli_stmt_close($stmt);
        }
    }

    public function productUrlSelect($prefix, $suffix) {
        $sql = "SELECT prefix, suffix, mime, `hash` 
                FROM frontier_products_v1 
                INNER JOIN frontier_urls_v1 
                ON frontier_products_v1.id = frontier_urls_v1.product_id 
                WHERE frontier_products_v1.prefix=? AND frontier_urls_v1.suffix=?";
        try {
            $stmt = mysqli_prepare($this->connection, $sql);
            mysqli_stmt_bind_param($stmt, 'ss', $prefix, $suffix);
            mysqli_stmt_execute($stmt);
            $result = mysqli_stmt_get_result($stmt);
            return mysqli_fetch_assoc($result);
        } catch (Exception $e) {
            throw new JsonPluginException('db-error', $e->getMessage());
        } finally {
            mysqli_stmt_close($stmt);
        }
    }

    public function urlRecordInsert($productId, $suffix, $mime, $hash) {
        $sql = "INSERT INTO frontier_urls_v1 (product_id, suffix, mime, `hash`) VALUES (?, ?, ?, ?)";
        try {
            $stmt = mysqli_prepare($this->connection, $sql);
            mysqli_stmt_bind_param($stmt, 'ssss', $productId, $suffix, $mime, $hash);
            return mysqli_stmt_execute($stmt);
        } catch (Exception $e) {
            throw new JsonPluginException('db-error', $e->getMessage());
        } finally {
            mysqli_stmt_close($stmt);
        }
    }

    public function urlRecordDelete($productId) {
        $sql = "DELETE FROM frontier_urls_v1 WHERE product_id=?";
        try {
            $stmt = mysqli_prepare($this->connection, $sql);
            mysqli_stmt_bind_param($stmt, 's', $productId);
            return mysqli_stmt_execute($stmt);
        } catch (Exception $e) {
            throw new JsonPluginException('db-error', $e->getMessage());
        } finally {
            mysqli_stmt_close($stmt);
        }
    }

    public function close() {
        mysqli_close($this->connection);
    }
}
