<?php
/*
 * API is at [root]/fcwfapi.php
 */ 

require_once(dirname(__FILE__) . '/db.php'); // DB wrapper
require_once(dirname(__FILE__) . '/util.php');
require_once(dirname(__FILE__) . '/jsonpluginexception.php'); // Custom JSON exception class

// Define allowed operations
define('OPS', [
    'getBlob',
    'uploadBlob', 
    'uploadProduct', 
    'deleteProduct', 
    'uploadEmergencyBar', 
    'deleteEmergencyBar',
]);
define('START_DATE', 1577836800); // 12:00am Jan 1, 2020
define('END_DATE', 4733510400); // 12:00am Jan 1, 2120

function getBlob($req) {
    $hash = $req['blobHash'];
    $mimeHex = $req['mime'];
    if (!$hash || !$mimeHex) {
        return header('HTTP/1.1 404 Not Found');
    }
    
    $db = new DbConnection();
    $blob = $db->blobSelect($hash);
    if (!$blob) {
        return header('HTTP/1.1 404 Not Found');
    }

    $mime = pack('H*', $mimeHex);
    header('HTTP/1.1 200 OK');
    header('Content-Type: ' . $mime);
    header('Content-Length: ' . strlen($blob));
    header('CacheControl: max-age:1'); // Cache for 1 second
    header('ETag: "' . $hash . '"');
    echo $blob;
    exit();
}


// `blob` contains the base64 encoded binary of the incoming file. Decode it, hash it and dump it 
// into the blobs DB table
function uploadBlob($req) {
    $blobB64 = $req['blob'];
    if (!$blobB64) {
        throw new JsonPluginException('invalid-request', 'Missing request data for uploadBlob');
    }

    try {
        $blob = base64_decode($blobB64);
    } catch (Exception $e) {
        throw new JsonPluginException('invalid-input', 'Invalid Base64 encoding');
    }
    
    try {
        $hash = md5($blob);
        $db = new DbConnection();
        $db->blobInsert($hash, $blob);
    } finally {
        $db->close();
    }

    return json_encode([
        'success' => true,
        'hash' => $hash
    ]);
}


// Dumps incoming JSON into the DB
function uploadProduct($req) {
    $prefix = $req['prefix'];
    $files = $req['files'];
    $type = $req['type'];
    if (!$prefix || !$files || !$type) {
        throw new JsonPluginException('invalid-request', 'Missing request data for uploadProduct');
    }

    $title = $req['title'] ?? '';
    $start = is_numeric($req['start']) ? intval($req['start']) : START_DATE;
    $end = is_numeric($req['end']) ? intval($req['end']) : END_DATE;

    try {
        $db = new DbConnection();
        $productId = md5($prefix);
        $db->productInsert($productId, $prefix, $title, $type, $start, $end);
        
        // Drop all URL records for the incoming prefix
        $db->urlRecordDelete($productId);

        $missingHashes = [];
        foreach ($files as $file) {
            list('suffix' => $suffix, 'mime' => $mime, 'hash' => $hash) = $file;
            // Check for all required file data
            if (!$suffix || !$mime || !$hash) {
                throw new JsonPluginException(
                    'invalid-request', 
                    'Missing file data for uploadProduct'
                );
            }

            // Refill URL records with the incoming JSON data
            $db->urlRecordInsert($productId, $suffix, $mime, $hash);
            // Check if blob exists for the given hash
            if(!$db->blobSelect($hash)) {
                $missingHashes[] = $hash;
            }
        }

        return json_encode([
            'success' => true,
            'missingHashes' => $missingHashes
        ]);
    } finally {
        $db->close();
    }
}

function deleteProduct($req) {
    $prefix = $req['prefix'];
    if (!$prefix) {
        throw new JsonPluginException('invalid-request', 'Missing prefix for deleteProduct');
    }

    try {
        $id = md5($prefix);
        $db = new DbConnection();
        // Deop all URL records that match the productId
        if ($db->urlRecordDelete($id)) {
            // Drop product entry if URL records are successfully dropped
            $db->productDelete($id);
        }
        return json_encode([ 'success' => true ]);
    } finally {
        $db->close();
    }
}

function uploadEmergencyBar($req) {
    $hash = $req['hash'];
    $start = $req['start'];
    $end = $req['end'];
    if (!$hash || !$start || !$end) {
        throw new JsonPluginException('invalid-request', 'Missing data for uploadEmergencyBar');
    }

    // Add fcwf_emergency_bar to wp_options table
    update_option('fcwf_emergency_bar', "$start $end $hash");
    try  {
        $db = new DbConnection();
        $missingHashes = [];
        // Check if blob exists for the given hash
        if(!$db->blobSelect($hash)) {
            $missingHashes[] = $hash;
        }
        return json_encode([ 
            'success' => true, 
            'missingHashes' => $missingHashes
        ]);
    } finally {
        $db->close();
    }
}

function deleteEmergencyBar($req) {
    update_option('fcwf_emergency_bar', null);
    return json_encode([ 'success' => true ]);
}

function main() {
    try {
        $op = $_GET['op'];
        $method = $_SERVER['REQUEST_METHOD'];
        if (!$op || !in_array($op, OPS)) {
            throw new JsonPluginException('invalid-op', 'Invalid operation');
        }

        if ($method == 'POST') {
            $rawArgs = file_get_contents('php://input');
            $args = json_decode($rawArgs, true);
            $secret = $args['sharedSecret'];
            $valid = checkSecret($secret);
            if (!$secret || !$valid) {
                throw new JsonPluginException(
                    'invalid-credentials',
                    'Missing or incorrect credentials'
                );
            }
            $result = $op($args);
            echo $result;
        } elseif ($method == 'GET' && $op == 'getBlob') {
            getBlob($_GET);
        } else {
            throw new JsonPluginException('invalid-request', "{$method} is not supported.");
        }
    } catch (JsonPluginException $e) {
        if ($e->obj['err'] == 'invalid-credentials') {
            header('HTTP/1.1 401 Unauthorized');
        }
        echo json_encode($e->obj);
    } catch (Exception $e) {
        header('HTTP/1.1 500 Internal Server Error');
        echo json_encode([
            'err' => 'php-error', 
            'msg' => $e->getMessage()
        ]);
    }
}


main();
