<?php
/*
 * Plugin Name:       Frontier Workflow
 * Description:       Frontier Live Workflow integration plugin.
 * Version:           1.0.0
 * Requires at least: 5.0 or greater
 * Requires PHP:      7.0
 * Author:            Frontier Marketing
 * Author URI:        https://frontier.io
 */

require_once(dirname(__FILE__) . '/db.php'); // DB wrapper
require_once(dirname(__FILE__) . '/util.php');

// Polyfill for PHP < 7.3
if (! function_exists("array_key_last")) {
    function array_key_last($array) {
        if (!is_array($array) || empty($array)) {
            return null;
        }
        return array_keys($array)[count($array)-1];
    }
}

register_activation_hook(__FILE__, 'activate_frontier_workflow');
function activate_frontier_workflow() {
    try {
        $db = new DbConnection();
        $db->dbTableCreate();
        $secret = generateSecret();
        // Send WP install config to Live if upsert is successful (ie. no secret in DB)
        if ($db->configInsert('liveSecret', $secret)) {
            $url = 'https://live.frontier.io/exec/workflow';
            $config = clientSiteConfig($secret);
            $config['op'] = 'unclaimedSitePut';
            $resp = httpPost($url, $config);
        }
    } finally {
        $db->close();
    }
}

add_action('init', 'fcwf_api_rewrite');
function fcwf_api_rewrite() {
    // API rewrite rule making API available at [root]/fcwfapi_api.php
    add_rewrite_rule('fcwfapi.php$', 'index.php?fcwfapi=1', 'top');
    flush_rewrite_rules();
}

add_filter('query_vars', 'fcwf_api_query_vars');
function fcwf_api_query_vars($query_vars) {
    $query_vars[] = 'fcwfapi';
    return $query_vars;
}

// Check if request matches files uploaded from Live, set headers, dump binary, and exit to 
// prevent WP from running the rest of it's hooks.
add_action('parse_request', 'fcwf_api_parse_request');
function fcwf_api_parse_request(&$wp) {
    if (array_key_exists('fcwfapi', $wp->query_vars)) {
        include 'fcwf-api.php';
        exit();
    }

    $path = getRequestPath($_SERVER);

    // Check for homepage redirect
    if ($path === '/') {
        if ($homepageRedirect = get_option('fcwf_homepage_redirect')) {
            $now = time();
            [$start, $end, $hash] = explode(' ', $homepageRedirect);
            if ($now >= $start && $now < $end) {
                $db = new DbConnection();
                $redirectJson = json_decode($db->blobSelect($hash), true);
                if ($redirectURL = $redirectJson['redirectURL']) {
                    header("Location: $redirectURL", true, 302);
                    exit();
                }
            }
        }
    }
    
    // Redirect to add trailing slash to URL
    $splitPath = preg_split('/\//', $path, NULL, PREG_SPLIT_NO_EMPTY);
    if (
        substr($path, -1) != '/' && 
        strpos($splitPath[array_key_last($splitPath)], '.') === false
    ) {
        $queryString = $_SERVER['QUERY_STRING'];
        $uri = $queryString ? $path . '/?' . $queryString : $path . '/';
        header('Location: ' . $uri, true, 302);
        exit();
    }

    $isPreview = isPreviewLink($path);
    if ($isPreview) {
        // Remove preview link section from path to continue matching as normal
        $path = getRequestPath($_SERVER, true);
    }
    
    $db = new DbConnection();
    $productData = getProductData($db, $path);
    // Exit hook if no product data is returned for current path
    if ($productData === false) return;
    
    // Since checking for prefix matches and getting product data may need to occur in multiple 
    // WP hooks, set product data as a global to eliminate these calls in other hooks.
    define('FCWF_PRODUCT_DATA', $productData);
    
    // This code path is only for 'static' pages and their assets. The 'isBaseProductPath' check is
    // needed since assets will not have a type.
    if ($productData['type'] == 'static' || !isBaseProductPath($productData['prefix'], $path)) {
        $now  = time();
        $start = $productData['start'];
        $end = $productData['end'];
        // Show/hide product based on it's start and end dates
        if ($isPreview || $start <= $now && $now <= $end)  {
            $suffix = getProductSuffix($productData['prefix'], $path);
            $file = getFileFromPath($db, $productData['prefix'], $suffix);
            $blob = $file['blob'];
            if ($blob) {
		        // Clear output buffer before sending headers/blob. GravityForms is echoing a 
                // <script> that was being included with our blob and rendering an incomplete 
                // HTML page.
                ob_end_clean();
	
                // If emergency bar is active, add a script to the <head> tag in index.html to 
                // render it.
                if (strpos($suffix, 'index.html') !== false) {
                    $pos = strpos($blob, '<head>');
                    $emergencyBarScript = getActiveEmergencyBarScript();
                    if ($pos !== false && $emergencyBarScript) {
                        $blob = substr($blob, 0, $pos) . $emergencyBarScript . substr($blob, $pos);
                    }
                }
                header('HTTP/1.1 200 OK');
                header('Content-Type: ' . $file['mime']);
                header('Content-Length: ' . strlen($blob));
                header('Cache-Control: max-age=1'); // Cache for 1 second
                header('ETag: "' . $file['hash'] . '"');
                echo $blob;
                exit();
            }
        }
    }
    return;
}

// Change page template for embedded products (products that use WP default header/footer)
add_filter('template_include', 'fcwf_template_include', 1);
function fcwf_template_include($template) {
    if (defined('FCWF_PRODUCT_DATA')) {
        $productData = FCWF_PRODUCT_DATA;
        if ($productData['type'] === 'embed') {
            $isPreview = isPreviewLink($path);
            $now  = time();
            $start = $productData['start'];
            $end = $productData['end'];
            
            // Show/hide product based on it's start and end dates
            if ($isPreview || $start <= $now && $now <= $end) {
                // Since embedded pages are not in the WP DB, the page will load with an HTTP 404 
                // status code. Set HTTP Status to 200 OK since the page does exist.
                status_header(200);
                header('Cache-Control: max-age=1'); // Cache for 1 second
                $template = dirname(__file__) . '/template-embed.php';
            }
        }
    }
    return $template;
}

// This is a custom hook that should only be called from template-embed.php. Get the blob for the 
// embedded page content, and render it between the WP-generated header and footer tags.
add_filter('fcwf_embed_main', 'fcwf_embed_main');
function fcwf_embed_main() {
    if (defined('FCWF_PRODUCT_DATA')) {
        $productData = FCWF_PRODUCT_DATA;
        $path = getRequestPath($_SERVER, true);
        $suffix = getProductSuffix($productData['prefix'], $path);
        
        $db = new DbConnection();
        $file = getFileFromPath($db, $productData['prefix'], $suffix);
        $blob = $file['blob'];
        if ($blob) {
            return $blob;
        }
    }
    return;
}

add_filter('fcwf_get_title', 'fcwf_get_title');
function fcwf_get_title() {
    if (defined('FCWF_PRODUCT_DATA')) {
        return '<title>' . FCWF_PRODUCT_DATA['title'] . '</title>';
    }
    return;
}

// Echo emergency bar script to every WP page (including Live-originating 'embedded' pages). This 
// hook just echoes the script into the HTML <head> tag. `wp_head` is a common hook that *should* 
// be called by every page in WP.
add_action('wp_head', 'fcwf_emergency_bar_script');
function fcwf_emergency_bar_script() {
    $emergencyBarScript = getActiveEmergencyBarScript();
    if ($emergencyBarScript) {
        echo $emergencyBarScript;
    }
}
