<?php

require_once(dirname(__FILE__) . '/db.php'); // DB wrapper

function generateSecret($length=36) {
    $chars = array_merge(range('A', 'Z'), range('a', 'z'), range(0, 9));
    $secret = '';
    for ($i = 0; $i < $length; $i++) {
        $randIdx = rand(0, count($chars));
        $secret .= $chars[$randIdx];
    }
    return $secret;
}

// Sent to Live API when plugin is activated to initialize the WP install
function clientSiteConfig($secret) {
    return [
        'type' => 'wordpress',
        'url' => get_site_url(), // WP function
        'sharedSecret' => $secret
    ];
}

function httpPost($url, $payload) {
    $curl = curl_init($url);
    curl_setopt($curl, CURLOPT_POST, true);
    curl_setopt($curl, CURLOPT_POSTFIELDS, json_encode($payload));
    curl_setopt($curl, CURLOPT_HTTPHEADER, array('Content-Type:application/json'));
    curl_setopt($curl, CURLOPT_RETURNTRANSFER, true); // Return response instead of printing
    $resp = curl_exec($curl);
    curl_close($curl);
    return $resp;
}

function checkSecret($secret) {
    try {
        $db = new DbConnection();
        $liveSecret = $db->configSelect('liveSecret');
        if (!$liveSecret) {
            return false;
        }

        return $secret == $liveSecret;
    } finally {
        $db->close();
    }
}

function getRequestPath($server, $stripPreview=false) {
    $requestUri = $server['REQUEST_URI'];
    $queryString = $server['QUERY_STRING'];
    $path = str_replace('?'.$queryString, '', $requestUri);
    if ($stripPreview) {
        $path = str_replace('/_preview_do_not_link_', '', $path);
    }
    return $path;
}

function getProductData($db, $path) {
    $products = $db->productSelectAll();
    $prefix = prefixMatch($path, array_keys($products));
    if (!$prefix) return false;
    $product = $products[$prefix];
    $product['prefix'] = $prefix;
    return $product;
}

function isPreviewLink($path) {
    return strpos($path, '_preview_do_not_link') !== false;
}

// Check if path ends with / or index.html
function isBaseProductPath($prefix, $path) {
    $pathSuffix = str_replace($prefix, '', $path);
    $splitSuffix = preg_split('/\//', $pathSuffix, NULL, PREG_SPLIT_NO_EMPTY);
    $last = $splitSuffix[array_key_last($splitSuffix)];
    return strpos($last, '.') === false || $last === 'index.html';
}

function prefixMatch($path, $prefixes) {
    if (count($prefixes) == 0) return false;
    // filter prefixes before sorting
    $filterPrefix = explode('/', trim($path, '/'))[0];
    $filtered = array_filter($prefixes, function($prefix) {
        return $filterPrefix == substr($prefix, 0, strlen($filterPrefix));
    });
    // order prefixes descending by number of slashes
    usort($filtered, function ($a, $b) {
        $aCount = substr_count($a, '/');
        $bCount = substr_count($b, '/');
        return $aCount < $bCount ? -1 : $aCount > $bCount ? 1 : 0;
    });
    $pathMatch = substr($path, 0, strlen($path));
    foreach ($filtered as $prefix) {
        if (strpos($pathMatch, '/'.$prefix.'/') === 0) {
            return $prefix;
        }
    }
    return false;
}

function getActiveEmergencyBarScript() {
    $emergencyBar = get_option('fcwf_emergency_bar');
    if ($emergencyBar) {
        $now = time();
        [$start, $end, $hash] = explode(' ', $emergencyBar);
        if ($now >= $start && $now < $end) {
            $db = new DbConnection();
            $ebJson = json_decode($db->blobSelect($hash), true);
            $excludedUrls = $ebJson['excludedUrls'];
            // Build request url (without query string) to compare with excluded urls
            $requestUrl = strtok("https://$_SERVER[HTTP_HOST]$_SERVER[REQUEST_URI]", "?");
            if (count($excludedUrls) && isUrlExcluded($requestUrl, $excludedUrls)) {
                return false;
            }
            $html = $ebJson['html'];
            return "<script type=\"text/javascript\">
                document.addEventListener('DOMContentLoaded', function() {
                    var div = document.createElement('div');
                    div.innerHTML = String(`$html`).trim();
                    document.body.appendChild(div.firstChild);
                    var height = document.querySelector('#fcwf-emergency-bar-wrapper').offsetHeight;
                    document.body.style.paddingTop = height + 'px';
                    document.body.style.position = 'relative';
                    document.body.style.margin = '0';
                });
            </script>";
        }
    }
    return false;
}

function isUrlExcluded($url, $excludedUrls) {
    function strStartsWith($str, $query) {
        return substr($str, 0, strlen($query)) === $query;
    }

    if (!$excludedUrls) return false;
    foreach($excludedUrls as $excludedUrl) {
        $asterixPos = strpos($excludedUrl, '*');
        // Exclude any request URL that starts with an excluded URL containing as asterix
        if ($asterixPos) {
            $excludedUrl = substr($excludedUrl, 0, $asterixPos);
            if (strStartsWith($url, $excludedUrl)) {
                return true;
            }
        }
        // Trim rightmost / to prevent mismatches
        if (rtrim($url, '/') === rtrim($excludedUrl, '/')) {
            return true;
        }
    }
    return false;
}

function getProductSuffix($prefix, $path) {
    $pos = strpos($path, $prefix);
    // Replace first occurance of $prefix
    $pathSuffix = substr_replace($path, '', $pos, strlen($prefix));
    $splitSuffix = preg_split('/\//', $pathSuffix, NULL, PREG_SPLIT_NO_EMPTY);
    // Add index.html to path if no file is specified in the path
    if (strpos($splitSuffix[array_key_last($splitSuffix)], '.') === false) {
        $splitSuffix[] = 'index.html';
    }
    return '/' . implode('/', $splitSuffix);
}

function getFileFromPath($db, $prefix, $suffix) {
    $file = $db->productUrlSelect($prefix, $suffix);
    if ($file) {
        $file['blob'] = $db->blobSelect($file['hash']);
        return $file;
    }
    return false;
}
